
import numpy as np


class ConfidenceBounds:
    def __init__(self, nr_weights, nr_samples=100, beta=1.2, seed=None, lower_bound=-1.0, upper_bound=1.0):
        self.nr_weights = nr_weights
        self.nr_samples = nr_samples
        self.beta = beta    # if beta negative => lower confidence bounds
        self.lower_bound = lower_bound
        self.upper_bound = upper_bound
        self.seed = seed

    def __call__(self, gauss_process, _, seed=None):
        # if seed is set for whole experiment
        if self.seed is not None:
            seed = self.seed

        # random generator
        rng = np.random.default_rng(seed)

        # sample from the surrogate
        x_test = rng.uniform(self.lower_bound, self.upper_bound, size=(self.nr_samples, self.nr_weights))
        mu, sigma = gauss_process.predict(x_test, return_std=True)

        # upper/lower confidence bounds
        cb = mu + self.beta * sigma

        # get the best result and return it
        idx = np.argmax(cb)
        return x_test[idx, :]
